﻿
using ProjectMonorail.Scripts.Exceptions;
using ProjectMonorail.Scripts.Monorail.Drawnings;
using System.Collections.Generic;
using System.Diagnostics;

namespace ProjectMonorail.Scripts.Monorail.CollectionGenericObjects
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
    public class ListGenericObjects<T> : ICollectionGenericObjects<T>
        where T : class
    {
        /// <summary>
        /// Список объектов, которые храним
        /// </summary>
        private readonly List<T?> _collection;

        /// <summary>
        /// Максимально допустимое число объектов в списке
        /// </summary>
        private int _maxCount;

        public int Count => _collection.Count;

        public int MaxCount { 
            get { return Count; } 
            set { if (value > 0) { _maxCount = value; } }
        }

        public CollectionType GetCollectionType => CollectionType.List;

        /// <summary>
        /// Конструктор
        /// </summary>
        public ListGenericObjects()
        {
            _collection = new();
        }

        public T? Get(int position)
        {
            // TODO проверка позиции
            if (position < 0 || position >= _collection.Count) return null;
            return _collection[position];
        }

        public int Insert(T obj, IEqualityComparer<T?>? comparer = null)
        {
            // TODO проверка, что не превышено максимальное количество элементов
            // TODO вставка в конец набора

            if (comparer != null)
            {
                foreach (T? item in _collection)
                {
                    if (item == null) continue;
                    if ((comparer as IEqualityComparer<DrawingMonorail>).Equals(obj as DrawingMonorail, item as DrawingMonorail))
                    {

                        throw new ObjectIsEqualException();
                    }
                }
            }

            _collection.Add(obj);
            return _collection.Count + 1;
        }

        public int Insert(T obj, int position, IEqualityComparer<T?>? comparer = null)
        {
            // TODO проверка, что не превышено максимальное количество элементов
            // TODO проверка позиции
            // TODO вставка по позиции

            if (comparer != null)
            {
                foreach (T? item in _collection)
                {
                    if ((comparer as IEqualityComparer<DrawingMonorail>).Equals(obj as DrawingMonorail, item as DrawingMonorail))
                    {
                        throw new ObjectIsEqualException();
                    }
                }
            }

            _collection.Insert(position, obj);

            return position;
        }

        public T Remove(int position)
        {
            // TODO проверка позиции
            // TODO удаление объекта из списка

            if (position < 0 || position > _collection.Count) throw new PositionOutOfCollectionException(position);

            T temp = _collection[position];

            _collection[position] = null;

            return temp;
        }

        public IEnumerable<T?> GetItems()
        {
            for (int i = 0; i < _collection.Count; ++i)
            {
                yield return _collection[i];
            }
        }

        void ICollectionGenericObjects<T>.CollectionSort(IComparer<T?> comparer)
        {
            _collection.Sort(comparer);
        }
    }
}
